
`timescale 1ns / 1ps

module circ_buf(CLK,RST,TRIGGER,IN_A,IN_B,SEL,ENA,READ_REQ,MT,DOUT);

    input CLK;
    input RST;
    input TRIGGER;
    input [15:0] IN_A;
    input [15:0] IN_B;
    input SEL;
    input ENA;
    input READ_REQ;
    output MT;
    output [15:0] DOUT;

reg READ_A;
reg READ_B;
reg RST_WRT_CNT;
reg WE;
reg MT;
reg SHFT_ST;
reg READ_A_REQ;
reg READ_B_REQ;

wire A_FULL;
wire B_FULL;
wire A_MT;
wire B_MT;
wire RWC;
wire WRT_DONE;
wire [9:0] WADDR;
wire [9:0] RADDR_A;
wire [9:0] RADDR_B;
wire [9:0] WRT_CNT;
wire [15:0] OUT_A;
wire [15:0] OUT_B;



`ifdef HEADER
`else
`include "header.v"
`endif

assign   A_FULL = (WADDR == ((RADDR_A-1)%1024));
assign   B_FULL = (WADDR == ((RADDR_B-1)%1024));
assign     A_MT = (WADDR == RADDR_A);
assign     B_MT = (WADDR == RADDR_B);
assign      RWC = RST_WRT_CNT || RST;
assign WRT_DONE = (WRT_CNT == `Cir_Buf_Term_Cnt);


///////////////////////////////////////////////////////////////
//							     //
// Circular buffer  (Size: 1k X 32 bits)	             //
// Two 16 bit inputs with common write address (write locked)//
// Two independant 16 bit outputs 			     //
//	(contains only the most recent results)		     //
//							     //
///////////////////////////////////////////////////////////////

 
defparam    cir_buf_wrt_ptr.width = 10;
defparam cir_buf_read_A_ptr.width = 10;
defparam cir_buf_read_B_ptr.width = 10;
defparam   cir_buf_wrt_cntr.width = 10;

cnt_up      cir_buf_wrt_ptr (.clk(CLK),.inc(WE),.r(RST),.count(WADDR));
dpm_1kX16         cir_buf_A (.clk(CLK),.we(WE),.a(WADDR),.dpra(RADDR_A),.di(IN_A),.dpo(OUT_A)); 
dpm_1kX16         cir_buf_B (.clk(CLK),.we(WE),.a(WADDR),.dpra(RADDR_B),.di(IN_B),.dpo(OUT_B)); 
cnt_up   cir_buf_read_A_ptr (.clk(CLK),.inc(READ_A),.r(RST),.count(RADDR_A));
cnt_up   cir_buf_read_B_ptr (.clk(CLK),.inc(READ_B),.r(RST),.count(RADDR_B));
cnt_up     cir_buf_wrt_cntr (.clk(CLK),.inc(WE),.r(RWC),.count(WRT_CNT));
mux2to1_reg_3st out_mux_reg (.clk(CLK),.sel(SEL),.ena(ENA),.ain(OUT_A),.bin(OUT_B),.out(DOUT));

always @ (READ_A_REQ or WE or A_FULL)
begin
   casex ({WE,A_FULL,READ_A_REQ})
      3'bxx1,3'b110:
         begin		  // increment read address
            READ_A <= 1'b1;
         end
      default:	          // hold
         begin
            READ_A <= 1'b0;
         end
   endcase
end
always @ (READ_B_REQ or WE or B_FULL)
begin
   casex ({WE,B_FULL,READ_B_REQ})
      3'bxx1,3'b110:
         begin		  // increment read address
            READ_B <= 1'b1;
         end
      default:	          // hold
         begin
            READ_B <= 1'b0;
         end
   endcase
end

always @ (posedge CLK or posedge RST)
begin:  Cir_Buf_Wrt_Ctrl_FSM
   if(RST)
      SHFT_ST <= `Wait_for_Trig;
   else
      case(SHFT_ST)
         `Wait_for_Trig:
	    if(TRIGGER)
	       SHFT_ST <= `Write_Cir_Buf;
            else
	       SHFT_ST <= `Wait_for_Trig;
         `Write_Cir_Buf:
	    if(WRT_DONE)
	       SHFT_ST <= `Wait_for_Trig;
            else
	       SHFT_ST <= `Write_Cir_Buf;
         default:
	       SHFT_ST <= `Wait_for_Trig;
      endcase
end

always @(SHFT_ST)
begin: Cir_Buf_Wrt_Ctrl_FSM_outs
   case(SHFT_ST)
      `Wait_for_Trig:
         begin
	    RST_WRT_CNT <= 1'b1;
	    WE <= 1'b0;
	 end
      `Write_Cir_Buf:
         begin
	    RST_WRT_CNT <= 1'b0;
	    WE <= 1'b1;
	 end
      default:
         begin
	    RST_WRT_CNT <= 1'b1;
	    WE <= 1'b0;
	 end
   endcase
end      

always @ (SEL or READ_REQ or A_MT or B_MT)
begin: Cir_Buf_Read_Ctrl
   if(SEL)
      begin
         MT <= B_MT;
         READ_A_REQ <= 0;
         READ_B_REQ <= READ_REQ;
      end
   else
      begin
         MT <= A_MT;
         READ_A_REQ <= READ_REQ;
         READ_B_REQ <= 0;
      end
end

endmodule
